@@ -35,6 +35,18 @@ def wait_on_sandbox(func, delay=0.1, max_wait=5.0):
     raise TronSandboxException("Failed %s" % func.__name__)
 
 
+def wait_on_state(client_func, url, state):
+    """Use client_func(url) to wait until the resource changes to state."""
+    def wait_func():
+        return client_func(url)['state'] == state
+    wait_func.__name__ = '%s wait on %s' % (url, state)
+    wait_on_sandbox(wait_func)
+
+
+def build_waiter_func(client_func, url):
+    return functools.partial(wait_on_state, client_func, url)
+
+
 def handle_output(cmd, (stdout, stderr), returncode):
     """Log process output before it is parsed. Raise exception if exit code
     is nonzero.
@@ -64,14 +76,23 @@ class SandboxTestCase(TestCase):
 
     _suites = ['sandbox']
 
+    sandbox = None
+
     @setup
     def make_sandbox(self):
+        verify_environment()
         self.sandbox = TronSandbox()
+        self.client = self.sandbox.client
 
     @teardown
     def delete_sandbox(self):
-        self.sandbox.delete()
-        self.sandbox = None
+        if self.sandbox:
+            self.sandbox.delete()
+            self.sandbox = None
+
+    def start_with_config(self, config):
+        self.sandbox.save_config(config)
+        self.sandbox.trond()
 
 
 class ClientProxy(object):
@@ -104,12 +125,17 @@ class ClientProxy(object):
         return functools.partial(self.wrap, attr)
 
 
+def verify_environment():
+    for env_var in ['SSH_AUTH_SOCK', 'PYTHONPATH']:
+        if not os.environ.get(env_var):
+            raise TronSandboxException("Missing $%s in test environment." % env_var)
+
+
 class TronSandbox(object):
     """A sandbox for running trond and tron commands in subprocesses."""
 
     def __init__(self):
         """Set up a temp directory and store paths to relevant binaries"""
-        self.verify_environment()
         self.tmp_dir        = tempfile.mkdtemp(prefix='tron-')
         cmd_path_func       = functools.partial(os.path.join, repo_root, 'bin')
         cmds                = 'tronctl', 'trond', 'tronfig', 'tronview'
@@ -139,16 +165,6 @@ class TronSandbox(object):
         with open(self.log_conf, 'w') as fh:
             fh.write(config.format(self.log_file))
 
-    def verify_environment(self):
-        ssh_sock = 'SSH_AUTH_SOCK'
-        msg = "Missing $%s in test environment."
-        if not os.environ.get(ssh_sock):
-            raise TronSandboxException(msg % ssh_sock)
-
-        path = 'PYTHONPATH'
-        if not os.environ.get(path):
-            raise TronSandboxException(msg % path)
-
     def delete(self):
         """Delete the temp directory and shutdown trond."""
         if os.path.exists(self.pid_file):
@@ -178,6 +194,12 @@ class TronSandbox(object):
         args = list(args) if args else []
         return self.run_command('tronctl', args + ['--server', self.api_uri])
 
+    def tronctl_start(self, name):
+        self.tronctl(['start', name])
+
+    def tronctl_stop(self, name):
+        self.tronctl(['stop', name])
+
     def tronview(self, args=None):
         args = list(args) if args else []
         args += ['--nocolor', '--server', self.api_uri]
@@ -197,4 +219,4 @@ class TronSandbox(object):
 
     def tronfig(self, config_content, name=schema.MASTER_NAMESPACE):
         args = ['--server', self.api_uri, name, '-']
-        return self.run_command('tronfig', args, stdin_lines=config_content)
+        return self.run_command('tronfig', args, stdin_lines=config_content)
\ No newline at end of file
