@@ -1,25 +1,20 @@
 import logging
-
 from twisted.python import log
-
 from tron import event
-from tron.utils import observer
 
 
 logger = logging.getLogger(__name__)
 
 
-class CrashReporter(observer.Observable):
+class CrashReporter(object):
     """Observer for twisted events that can send emails on crashes
 
     Based on twisted.log.PythonLoggingObserver
     """
 
-    def __init__(self, emailer, mcp):
-        super(CrashReporter, self).__init__()
-        self.emailer = emailer
-        self.event_recorder = event.EventRecorder(
-            self, parent=mcp.event_recorder)
+    def __init__(self, emailer):
+        self.emailer        = emailer
+        self.event_recorder = event.get_recorder(str(self))
 
     def _get_level(self, event_dict):
         """Returns the logging level for an event."""
@@ -43,15 +38,18 @@ class CrashReporter(observer.Observable):
             return
 
         try:
-            self.event_recorder.emit_critical("crash", msg=text)
+            self.event_recorder.critical("crash", msg=text)
             self.emailer.send(text)
 
         except Exception:
             logger.exception("Error sending notification")
-            self.event_recorder.emit_critical("email_failure", msg=text)
+            self.event_recorder.critical("email_failure", msg=text)
 
     def start(self):
         log.addObserver(self.emit)
 
     def stop(self):
         log.removeObserver(self.emit)
+
+    def __str__(self):
+        return 'CrashReporter'
