@@ -2,7 +2,7 @@ import struct
 import logging
 
 from twisted.internet import defer
-from twisted.conch.ssh import channel, common
+from twisted.conch.ssh import channel, common, keys
 from twisted.conch.ssh import connection
 from twisted.conch.ssh import transport
 from twisted.conch.client import default
@@ -21,6 +21,36 @@ class ChannelClosedEarlyError(Error):
     pass
 
 
+class SSHAuthOptions(object):
+    """An options class which can be used by NoPasswordAuthClient. This supports
+    the interface provided by: twisted.conch.client.options.ConchOptions.
+    """
+    def __init__(self, identitys, use_agent):
+        self.use_agent = use_agent
+        self.identitys = identitys
+
+    @classmethod
+    def from_config(cls, ssh_config):
+        return cls(ssh_config.identities, ssh_config.agent)
+
+    def __getitem__(self, item):
+        if item != 'noagent':
+            raise KeyError(item)
+        return not self.use_agent
+
+    def __eq__(self, other):
+        return other and (
+            self.use_agent == other.use_agent and
+            self.identitys == other.identitys)
+
+    def __ne__(self, other):
+        return not self == other
+
+    def __str__(self):
+        context = self.__class__.__name__, self.identitys, self.use_agent
+        return "%s(%s, %s)" % context
+
+
 class NoPasswordAuthClient(default.SSHUserAuthClient):
     """Only support passwordless auth."""
     preferredOrder              = ['publickey']
@@ -28,26 +58,32 @@ class NoPasswordAuthClient(default.SSHUserAuthClient):
     auth_keyboard_interactive   = None
 
 
-
 class ClientTransport(transport.SSHClientTransport):
 
     connection_defer = None
 
-    def __init__(self, *args, **kwargs):
-        # These silly twisted classes tend not to have init functions, and
-        # they're all old style classes
-        # transport.SSHClientTransport.__init__(self, *args, **kwargs)
+    def __init__(self, username, options, expected_pub_key):
+        self.username         = username
+        self.options          = options
+        self.expected_pub_key = expected_pub_key
 
-        self.username = kwargs['username']
-        self.options = kwargs['options']
+    def verifyHostKey(self, public_key, fingerprint):
+        if not self.expected_pub_key:
+            return defer.succeed(1)
 
-    def verifyHostKey(self, pubKey, fingerprint):
-        # TODO: this should verify
-        return defer.succeed(1)
+        if self.expected_pub_key == keys.Key.fromString(public_key):
+            return defer.succeed(2)
+
+        msg = "Public key mismatch got %s expected %s" % (
+            fingerprint, self.expected_pub_key.fingerprint())
+        log.error(msg)
+        return defer.fail(ValueError(msg))
 
     def connectionSecure(self):
         conn = ClientConnection()
+        # TODO: this should be initialized by the ClientConnection constructor
         conn.service_defer = defer.Deferred()
+        # TODO: this should be initialized by the constructor
         self.connection_defer.callback(conn)
 
         auth_service = NoPasswordAuthClient(self.username, self.options, conn)
