@@ -1,33 +1,17 @@
-"""Time utilites"""
+"""Functions for working with dates and timestamps."""
 from __future__ import division
 import datetime
 import re
 import time
 
 
-
-# Global time override
-_current_time_override = None
-
-
-def override_current_time(current_time):
-    """Interface for overriding the current time
-
-    This is a test hook for forcing the app to think it's a specific time.
-    """
-    global _current_time_override
-    _current_time_override = current_time
-
-
 def current_time():
-    """Standard interface for generating the current time."""
-    if _current_time_override is not None:
-        return _current_time_override
-    else:
-        return datetime.datetime.now()
+    """Return the current datetime."""
+    return datetime.datetime.now()
 
 
 def current_timestamp():
+    """Return the current time as a timestamp."""
     return to_timestamp(current_time())
 
 
@@ -35,11 +19,13 @@ def to_timestamp(time_val):
     """Generate a unix timestamp for the given datetime instance"""
     return time.mktime(time_val.timetuple())
 
+
 def delta_total_seconds(td):
-    """Equivalent to timedelta.total_seconds() which is only available in 2.7.
+    """Equivalent to timedelta.total_seconds() available in Python 2.7.
     """
-    return (
-       td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6) / 10**6
+    microseconds, seconds, days = td.microseconds, td.seconds, td.days
+    return (microseconds + (seconds + days * 24 * 3600) * 10**6) / 10**6
+
 
 def macro_timedelta(start_date, years=0, months=0, days=0):
     """Since datetime doesn't provide timedeltas at the year or month level,
@@ -110,4 +96,4 @@ class DateArithmetic(object):
             return int(to_timestamp(dt)) + delta
 
         if attr == 'daynumber':
-            return dt.toordinal() + delta
+            return dt.toordinal() + delta
\ No newline at end of file
